package com.jeecms.resource.domain;

import com.jeecms.common.annotation.EncryptField;
import com.jeecms.common.base.domain.AbstractDomain;
import com.jeecms.common.exception.GlobalException;
import com.jeecms.common.exception.UploadExceptionInfo;
import com.jeecms.common.exception.error.SysOtherErrorCodeEnum;
import com.jeecms.common.upload.AliOssCloudClient;
import com.jeecms.common.upload.AmaZonS3OssUpload;
import com.jeecms.common.upload.MinioUtil;
import com.jeecms.common.upload.QcoscloudClient;
import com.jeecms.common.upload.QiniuOssCloudClient;
import com.jeecms.common.upload.UploadUtils;
import com.jeecms.common.upload.fastdfs.FastDFSClient;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.persistence.Cacheable;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.TableGenerator;
import javax.persistence.Transient;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * 云存储domain
 *
 * @author: tom
 * @date: 2019年3月5日 下午4:48:37
 * @Copyright: 杭州藏茗山科技有限公司 All rights reserved.Notice 仅限于授权后使用，禁止非授权传阅以及私自用于商业目的。
 */
@Cacheable
@Entity
@Table(name = "jc_sys_oss")
public class UploadOss extends AbstractDomain<Integer> implements Serializable {
    private static final Logger log = LoggerFactory.getLogger(UploadOss.class);

    public static final Short OSS_TYPE_Q_CLOUD = 1;

    public static final Short OSS_TYPE_QINIU_CLOUD = 3;

    public static final Short OSS_TYPE_ALI_CLOUD = 2;

    public static final Short OSS_TYPE_AMAZON_S3 = 4;

    public static final Short OSS_TYPE_MINIO = 5;
    public static final Short OSS_TYPE_FASTDFS = 6;

    public static final int OSS_UPLOAD_SUCCESS = 1;

    public static final int OSS_UPLOAD_FAIL = 0;

    public enum OSSTYPE {
        /**
         * 腾讯云cos
         */
        QCLOUD,
        /**
         * 七牛云oss
         */
        QINIUCLOUD,
        /**
         * 阿里云oss
         */
        ALICLOUD,
        /**
         * 百度云
         */
        BAIDUCLOUD,
        /**
         * 亚马逊云存储
         */
        AMAZONS3,
        /**
         * fastdfs
         */
        FASTDFS,
        /**
         * minio
         */
        MINIO
    }

    ;

    /**
     * 上传文件
     *
     * @param path 上传路径
     * @param ext  格式
     * @param in   输入流
     * @return String
     * @throws IOException     IOException
     * @throws GlobalException GlobalException
     * @Title: storeByExt
     */
    @Transient
    public String storeByExt(String path, String ext, InputStream in) throws IOException, GlobalException {
        String remoteFileName = UploadUtils.generateFilename(path, ext);
        String fileOssUrl = store(remoteFileName, in, null, -1);
        return fileOssUrl;
    }

    @Transient
    public String storeByExt(String path, String ext, InputStream in, String contentType, long size) throws IOException, GlobalException {
        String remoteFileName = UploadUtils.generateFilename(path, ext);
        String fileOssUrl = store(remoteFileName, in, contentType, size);
        return fileOssUrl;
    }

    @Transient
    public String storeByFilename(String filename, InputStream in, String contentType, long size) throws IOException, GlobalException {
        String fileOssUrl = store(filename, in, contentType, size);
        return fileOssUrl;
    }

    @Transient
    public String storeByFilename(String filename, InputStream in) throws IOException, GlobalException {
        String fileOssUrl = store(filename, in, null, -1);
        return fileOssUrl;
    }

    @Transient
    private String store(String remote, InputStream in, String contentType, long size) throws GlobalException {
        String result = "";
        String fileUrl = "";
        /**云存储需要/开头*/
        if (!remote.startsWith("/") && !OSS_TYPE_MINIO.equals(getOssType())) {
            remote = "/" + remote;
        }
        if (OSS_TYPE_Q_CLOUD.equals(getOssType())) {
            /**
             * 腾讯云 返回服务器的文件地址
             */
            result = QcoscloudClient.uploadFileByInputStream(getBucketArea(), getBucketName(), getAppId(),
                    getSecretId(), getAppKey(), remote, in);
            // 接口调用返回服务器的路径地址
            if(StringUtils.isNotEmpty(result)){
                fileUrl = getAccessDomain() + result;
                if(log.isInfoEnabled()){
                    log.info("上传文件腾讯云地址 fileUrl->{}",fileUrl);
                }
            }
        } else if (OSS_TYPE_ALI_CLOUD.equals(getOssType())) {
            /**
             * 阿里云
             */
            boolean succ = AliOssCloudClient.uploadFileByInputStream(getBucketName(), getEndPoint(),
                    getAppKey(), getSecretId(), remote, in);
            if (succ) {
                fileUrl = this.getAccessDomain() + remote;
            } else {
                log.error("upload error");
                throw new GlobalException(new UploadExceptionInfo(
                        SysOtherErrorCodeEnum.UPLOAD_OSS_ERROR.getDefaultMessage(),
                        SysOtherErrorCodeEnum.UPLOAD_OSS_ERROR.getCode()));
            }
        } else if (OSS_TYPE_QINIU_CLOUD.equals(getOssType())) {
            /**
             * 七牛云
             */
            boolean succ = QiniuOssCloudClient.uploadFileByInputStream(
                    getBucketArea(), getBucketName(),
                    getAppKey(), getSecretId(), remote, in);
            if (succ) {
                // 七牛云存存储accessDomain得设置带http://
                fileUrl = this.getAccessDomain() + remote;
            } else {
                throw new GlobalException(new UploadExceptionInfo(
                        SysOtherErrorCodeEnum.UPLOAD_OSS_ERROR.getDefaultMessage(),
                        SysOtherErrorCodeEnum.UPLOAD_OSS_ERROR.getCode()));
            }
        } else if (OSS_TYPE_AMAZON_S3.equals(getOssType())) {
            /**
             * 亚马逊云存储
             */
            String url = AmaZonS3OssUpload.upload(bucketName, endPoint, in, getAppKey(), getSecretId(), remote, getBucketArea());
            if (url == null) {
                throw new GlobalException(new UploadExceptionInfo(
                        SysOtherErrorCodeEnum.UPLOAD_OSS_ERROR.getDefaultMessage(),
                        SysOtherErrorCodeEnum.UPLOAD_OSS_ERROR.getCode()));
            } else {
                fileUrl = url.split("\\?")[0];
            }

        } else if (OSS_TYPE_MINIO.equals(getOssType())) {
            if (remote.startsWith("/")) {
                remote = remote.substring(1);
            }
            // xiaohui 20220524
            //contentType = new MimetypesFileTypeMap().getContentType(in);
            MinioUtil.upload(endPoint, appKey, secretId, bucketName, in, size, remote, contentType);
            //fileUrl = MinioUtil.getUrl(endPoint, appKey, secretId, bucketName, remote);
            if (endPoint.endsWith("/")) {
                fileUrl = endPoint + bucketName + "/" + remote;
            } else {
                fileUrl = endPoint + "/" + bucketName + "/" + remote;
            }
        } else if (OSS_TYPE_FASTDFS.equals(getOssType())) {
            /**
             * fastdfs
             */
            //读取nginx地址
            String ngUrl = getAccessDomain();
            FastDFSClient fastDFSClient = new FastDFSClient();
            try {
                fileUrl += fastDFSClient.upload(in, remote, null);
            } catch (Exception e) {
                e.printStackTrace();
            }
            // 加上url前缀
            fileUrl = ngUrl + "/" + fileUrl;
        }
        return fileUrl;
    }

    /**
     * 删除文件
     *
     * @param remote 远程文件路径
     * @Title: deleteFile
     * @return: boolean
     */
    @Transient
    public boolean deleteFile(String remote) {
        String result = "";
        /**云存储需要/开头*/
        if (!remote.startsWith("/") && !OSS_TYPE_MINIO.equals(getOssType())) {
            remote = "/" + remote;
        }
        if (OSS_TYPE_Q_CLOUD.equals(getOssType())) {
            result = QcoscloudClient.deleteFile(getBucketArea(), getBucketName(), getAppId(), getSecretId(),
                    getAppKey(), remote);
            if(StringUtils.isEmpty(result)){
                return true;
            }else {
                return false;
            }
        } else if (OSS_TYPE_ALI_CLOUD.equals(getOssType())) {
            return AliOssCloudClient.deleteFile(getBucketName(), getEndPoint(), getAppKey(),
                    getSecretId(), remote);
        } else if (OSS_TYPE_QINIU_CLOUD.equals(getOssType())) {
            return QiniuOssCloudClient.deleteFile(getBucketName(),
                    getAppKey(), getSecretId(), remote);
        } else if (OSS_TYPE_AMAZON_S3.equals(getOssType())) {
            return AmaZonS3OssUpload.deleteFile(getBucketName(), getEndPoint(), getAppKey(), getSecretId(), remote, getBucketArea());
        } else if (OSS_TYPE_MINIO.equals(getOssType())) {
            if (remote.startsWith("/")) {
                remote = remote.substring(1);
            }
            MinioUtil.delete(endPoint, appKey, secretId, bucketName, remote);
        } else if (OSS_TYPE_FASTDFS.equals(getOssType())) {
            FastDFSClient fastDFSClient = new FastDFSClient();
            int flag = 0;
            try {
                if (remote.startsWith("/")) {
                    remote = remote.substring(1);
                }
                flag = fastDFSClient.deleteFile(remote);
            } catch (Exception e) {
                e.printStackTrace();
            }
            return flag == 0;
        }
        return false;
    }

    /**
     * 得到某个文件夹下文件列表,不带后缀
     *
     * @param remote 远程文件路径
     * @Title: child
     * @return: List
     */
    @Transient
    public List<String> child(String remote) {
        List<String> list = new ArrayList<>(16);
        if (OSS_TYPE_Q_CLOUD.equals(getOssType())) {
            return QcoscloudClient.parseList(getBucketArea(), getBucketName(), getAppId(), getSecretId(),
                    getAppKey(), remote);
        } else if (OSS_TYPE_ALI_CLOUD.equals(getOssType())) {
            return AliOssCloudClient.parseList(getBucketName(), getEndPoint(), getAppKey(),
                    getSecretId(), remote.substring(1));
        } else if (OSS_TYPE_QINIU_CLOUD.equals(getOssType())) {
            return QiniuOssCloudClient.parseList(getBucketName(),
                    getAppKey(), getSecretId(), remote.substring(1));
        } else if (OSS_TYPE_AMAZON_S3.equals(getOssType())) {
            return AmaZonS3OssUpload.parseList(getBucketName(), remote, getBucketArea());
        } else if (OSS_TYPE_MINIO.equals(getOssType())) {
            return MinioUtil.list(getEndPoint(), appKey, secretId, bucketName, remote);
        }
        return list;
    }

    private static final long serialVersionUID = 1L;
    private Integer id;
    @EncryptField
    private String accessDomain;
    private String appId;
    private String appKey;
    private String bucketArea;
    private String bucketName;
    private String endPoint;
    private String ossName;
    private Short ossType;
    private String secretId;

    public UploadOss() {
    }

    @Id
    @Column(name = "id", unique = true, nullable = false)
    @TableGenerator(table = "hibernate_sequences", name = "tg_gou_oss", pkColumnValue = "gou_oss", initialValue = 0, allocationSize = 10)
    @GeneratedValue(strategy = GenerationType.TABLE, generator = "tg_gou_oss")
    @Override
    public Integer getId() {
        return this.id;
    }

    @Override
    public void setId(Integer id) {
        this.id = id;
    }

    @Column(name = "access_domain", nullable = false, length = 255)
    public String getAccessDomain() {
        return this.accessDomain;
    }


    public void setAccessDomain(String accessDomain) {
        this.accessDomain = accessDomain;
    }

    @Column(name = "app_id", length = 255)
    public String getAppId() {
        return this.appId;
    }

    public void setAppId(String appId) {
        this.appId = appId;
    }

    @Column(name = "app_key", nullable = false, length = 255)
    public String getAppKey() {
        return this.appKey;
    }

    public void setAppKey(String appKey) {
        this.appKey = appKey;
    }

    @Column(name = "bucket_area", length = 255)
    public String getBucketArea() {
        return this.bucketArea;
    }

    public void setBucketArea(String bucketArea) {
        this.bucketArea = bucketArea;
    }

    @Column(name = "bucket_name", length = 255)
    public String getBucketName() {
        return this.bucketName;
    }

    public void setBucketName(String bucketName) {
        this.bucketName = bucketName;
    }

    @Column(name = "end_point", length = 255)
    public String getEndPoint() {
        return this.endPoint;
    }

    public void setEndPoint(String endPoint) {
        this.endPoint = endPoint;
    }

    @Column(name = "oss_name", nullable = false, length = 255)
    public String getOssName() {
        return this.ossName;
    }

    public void setOssName(String ossName) {
        this.ossName = ossName;
    }

    @Column(name = "oss_type", nullable = false)
    public Short getOssType() {
        return this.ossType;
    }

    public void setOssType(Short ossType) {
        this.ossType = ossType;
    }

    @Column(name = "secret_id", nullable = false, length = 255)
    public String getSecretId() {
        return this.secretId;
    }

    public void setSecretId(String secretId) {
        this.secretId = secretId;
    }

}